rm(list=ls())

#variable numrique, remplacement des NA par la moyenne
traiter.numeric <- function(x){
 y <- x
 z <- na.omit(y)
 if (length(z) < length(y)){
  m <- mean(z)
  y[is.na(y)] <- m
 }
 return(y)
}

#variable catgorielle, remplacement des NA par le mode
traiter.discrete <- function(x){
  y <- x
  z <- na.omit(y)
  if (length(z) < length(y)){    
    frequence <- table(z)
    m <- which.max(frequence)
    y[is.na(y)] <- levels(x)[m]
  }
  return(y)
}

#traiter les donnes manquantes
traiter_missing_data <- function(x){
 if (is.factor(x) == T){
  return(traiter.discrete(x))
 } else {
  return(traiter.numeric(x))
 }
}

#construction matrice de confusion  partir du modle
#new.dataset reprsente l'chantillon test
#model est le modle
pred_and_confusion_matrix <- function(new.dataset,model){
  #probabilit prdit par le modle
  data.pred.prob <- predict(model,newdata = new.dataset)
  #affectation
  data.pred.class <- ifelse(data.pred.prob > 0.5,"B","A")
  data.pred.class <- as.factor(data.pred.class)
  #matrice de confusion (classe observe vs. prdite)
  mc <- table(new.dataset$classe,data.pred.class)
  print(mc)
  #taux de bon classement
  ca <- (mc[1,1]+mc[2,2])/sum(mc)
  print(ca)  
}

#chargement du fichier test
setwd(".")
data.test <- read.table(file="credit-german-test.txt",dec=".",sep="\t",header=T)

#chargement des donnes d'apprentissage avec les valeurs manquantes
data.train <- read.table(file="credit-german-md-20_00.txt",dec=".",sep="\t",header=T,na.strings="?")
summary(data.train)

#premire solution : listwise deletion
data.train.omitted <- na.omit(data.train)
#summary(data.train.omitted)
print(paste('Remaining observations : ',as.character(nrow(data.train.omitted))))
model.omitted <- glm(classe ~ ., family = binomial, data = data.train.omitted)
#valuation
pred_and_confusion_matrix(data.test,model.omitted)

#seconde solution : imputation univarie (moyenne, mode)
data.train.imputed <- as.data.frame(lapply(data.train,traiter_missing_data))
#summary(data.train.imputed)
model.imputed <- glm(classe ~ ., family = binomial, data = data.train.imputed)
#valuation
pred_and_confusion_matrix(data.test,model.imputed)
