rm(list=ls())

#variable numrique, remplacement des NA par la moyenne
traiter.numeric <- function(x){
 y <- x
 z <- na.omit(y)
 if (length(z) < length(y)){
  m <- mean(z)
  y[is.na(y)] <- m
 }
 return(y)
}

#variable catgorielle, remplacement des NA par le mode
traiter.discrete <- function(x){
  y <- x
  z <- na.omit(y)
  if (length(z) < length(y)){    
    frequence <- table(z)
    m <- which.max(frequence)
    y[is.na(y)] <- levels(x)[m]
  }
  return(y)
}

#traiter les donnes manquantes
traiter_missing_data <- function(x){
 if (is.factor(x) == T){
  return(traiter.discrete(x))
 } else {
  return(traiter.numeric(x))
 }
}

#construction matrice de confusion  partir du modle
#new.dataset reprsente l'chantillon test
#model est le modle
pred_and_confusion_matrix <- function(new.dataset,model){
  #probabilit prdit par le modle
  data.pred.prob <- predict(model,newdata = new.dataset)
  #affectation
  data.pred.class <- ifelse(data.pred.prob > 0.5,"B","A")
  data.pred.class <- as.factor(data.pred.class)
  #matrice de confusion (classe observe vs. prdite)
  mc <- table(new.dataset$CLASSE,data.pred.class)
  print(mc)
  #taux de bon classement
  ca <- (mc[1,1]+mc[2,2])/sum(mc)
  print(ca)  
}


setwd(".")

wave.test <- read.table(file="wave-test.txt",dec=".",sep="\t",header=T)

wave.train <- read.table(file="wave-train-md.txt",dec=".",sep="\t",header=T,na.strings="?")
print(nrow(wave.train))

wave.train.omitted <- na.omit(wave.train)
print(nrow(wave.train.omitted))
wave.model.omitted <- glm(CLASSE ~ ., family = binomial, data = wave.train.omitted)

pred_and_confusion_matrix(wave.test,wave.model.omitted)

wave.train.imputed <- as.data.frame(lapply(wave.train,traiter.numeric))
wave.model.imputed <- glm(CLASSE ~ ., family = binomial, data = wave.train.imputed)

pred_and_confusion_matrix(wave.test,wave.model.imputed)

